@@ -54,31 +54,63 @@ _last_deferfunc = None
 _attempting = None
 
 class deferfunc(object):
+    
+    deepArgHandler = {}
 
-    def __init__(self, f):
+    def __init__(self, f, deepTypes=[]):
         self.function = f
         self.errbacks = []
         self.error_handled = False
         self.deferred = None
+        self.deepTypes = deepTypes
+    
+    def handleArg(self, arg):
+        if isinstance(arg, defer.Deferred):
+            self.defers.append(arg)
+            return arg
+        elif self.deepTypes:
+            d = self.handleDeepArg(arg)
+            if d is arg:
+                return defer.succeed(arg)
+        else:
+            return defer.succeed(arg)
+
+    def handleDeepArg(self, arg):
+        for t in self.deepTypes:
+            if issubclass(type(arg), t):
+                return self.deepArgHandler[t](self, arg)
+        return arg
+    
+    def handleDeepList(self, arg):
+        for i, a in enumerate(arg):
+            arg[i] = self.handleArg(a)
+            self.defers.append(arg[i])
+            acb=arg[i].addCallback
+            @acb
+            def update(r):
+                arg[i] = r
+        return arg
+    deepArgHandler[list] = handleDeepList
 
     def __call__(self, *args, **kwargs):
         args = list(args)
-        defers = []
+        self.defers = []
         f = self.function
+
         for i, arg in enumerate(args):
-            if isinstance(arg, defer.Deferred):
-                defers.append(arg)
-                @arg.addCallback
-                def setResultingArg(r):
-                    args[i] = r
+            d=self.handleArg(arg)
+            @d.addCallback
+            def update(r):
+                args[i] = r
+
         for k, v in kwargs.items():
-            if isinstance(v, defer.Deferred):
-                defers.append(v)
-                @v.addCallback
-                def setResultingArg(r):
-                    kwargs[k] = r
-        if defers:
-            d = defer.DeferredList(defers, fireOnOneErrback=True, consumeErrors=True)
+            d=self.handleArg(v)
+            @d.addCallback
+            def setResultingArg(r):
+                kwargs[k] = r
+
+        if self.defers:
+            d = defer.DeferredList(self.defers, fireOnOneErrback=True, consumeErrors=True)
             @d.addCallback
             def callWithResults(_results):
                 return f(*args, **kwargs)
@@ -87,7 +119,7 @@ class deferfunc(object):
         self.deferred = d
         for errback in self.errbacks:
             d.addErrback(errback)
-        if not defers:
+        if not self.defers:
             try:
                 r = f(*args, **kwargs)
                 d.callback(r)
@@ -99,11 +131,16 @@ class deferfunc(object):
             d.chainDeferred(cleanup_d)
         return d
 
-def deferargs(f):
-    new_f = deferfunc(f)
-    global _last_deferfunc
-    _last_deferfunc = new_f
-    return new_f
+def deferargs(*args, **kwargs):
+    if len(args)==1 and isinstance(args[0], type(deferargs)):
+        new_f = deferfunc(args[0], **kwargs)
+        global _last_deferfunc
+        _last_deferfunc = new_f
+        return new_f
+    else:
+        deepTypes = kwargs.setdefault('deepTypes', [])
+        dec = lambda f: deferargs(f, deepTypes=deepTypes)
+        return dec
 
 
 def catch(errtype=None):
@@ -138,6 +175,10 @@ def attempt(f):
     return f
 
 
+def deferargscall(f, *args, **kwargs):
+    return deferargs(f)(*args, **kwargs)
+
+
 def cleanup(f):
     global _last_deferfunc
     if hasattr(_last_deferfunc.deferred, 'result'):
@@ -252,4 +293,14 @@ if __name__ == '__main__':
     this()
     reactor.runUntilCurrent()
     assert assert_caught, assert_caught
+    
+    def f(a):
+        assert a == 10
+    deferargscall(f, defer.succeed(10))
+    
+    @deferargs(deepTypes=[list])
+    def test(lst):
+        assert lst == [1,2], lst
+    test([1, defer.succeed(2)])
+    reactor.runUntilCurrent()
 
